﻿/*
A registry reading plugin to LCD Smartie by Alex Vallat (AlexV@ComPorts.com)
Version 2
Released under the GPL.
Provided As Is, with no warranties of any kind.
*/
#region Using directives

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;

#endregion

namespace RegistryReader
{
	public class LCDSmartie
	{
		public LCDSmartie()
		{

		}
		/// <summary>
		/// LCD Smartie plugin function call 1.
		/// Read the specified value from the registry.
		/// </summary>
		/// <param name="param1">The path to the registry key</param>
		/// <param name="param2">The name of the value to read</param>
		/// <returns>The value in the registry key specified</returns>
		public string function1(string param1, string param2)
		{
			ParseResults input = ParseInput(param1, param2);
			if (input.Success)
			{
				try
				{
					using (RegistryKey key = input.Hive.OpenSubKey(input.KeyName))
					{
						if (key == null) return "Could not find key named: " + input.KeyName;
						string value = (string)key.GetValue(input.ValueName);
						if (value == null) return "Could not find value named: " + input.ValueName;
						return value;
					}
				}
				catch (Exception ex)
				{
					return "Could not read value: " + ex.Message;
				}
			}
			else
			{
				return input.ErrorString;
			}
		}
		/// <summary>
		/// LCD Smartie plugin function call 2.
		/// Does the specified registry value exist?
		/// </summary>
		/// <param name="param1">The path to the registry key</param>
		/// <param name="param2">The name of the value to check</param>
		/// <returns>"1" if the key and value exists, "0" if it doesn't</returns>
		public string function2(string param1, string param2)
		{
			ParseResults input = ParseInput(param1, param2);
			if (input.Success)
			{
				try
				{
					using (RegistryKey key = input.Hive.OpenSubKey(input.KeyName))
					{
						if (key == null) return "0"; //Key not found
						string value = (string)key.GetValue(input.ValueName, null);
						if (value == null) return "0";
						return "1";
					}
				}
				catch (Exception ex)
				{
					return "Could not read value: " + ex.Message;
				}
			}
			else
			{
				return input.ErrorString;
			}
		}

		/// <summary>
		/// LCD Smartie plugin function call 3.
		/// Has the specified registry value changed?
		/// </summary>
		/// <param name="param1">The path to the registry key</param>
		/// <param name="param2">The name of the value to check</param>
		/// <returns>"1" if the key and value has changed since the last time this function was called, "0" if it hasn't</returns>
		public string function3(string param1, string param2)
		{
			ParseResults input = ParseInput(param1, param2);
			if (input.Success)
			{
				try
				{
					using (RegistryKey key = input.Hive.OpenSubKey(input.KeyName))
					{
						string value = null;
						if (key != null)
						{
							value = (string)key.GetValue(input.ValueName, null);
						}
						bool changed = value != mLastCheckedValue;
						mLastCheckedValue = value;
						
						return changed ? "1" : "0";
					}
				}
				catch (Exception ex)
				{
					return "Could not read value: " + ex.Message;
				}
			}
			else
			{
				return input.ErrorString;
			}
		}
		private string mLastCheckedValue;

		private ParseResults ParseInput(string path, string valueName)
		{
			if (String.IsNullOrEmpty(path))
			{
				return new ParseResults("No key specified");
			}

			if (valueName == null)
				valueName = "";
			
			int splitPos = path.IndexOf(@"\");
			if (splitPos < 0)
			{
				return new ParseResults("Invalid key format");
			}

			string hiveName = path.Substring(0, splitPos);
			string keyName = path.Substring(splitPos + 1);

			RegistryKey hive = GetHive(hiveName);
			if (hive == null)
			{
				return new ParseResults("Did not recognize top level hive name: " + hiveName);
			}

			return new ParseResults(hive, keyName, valueName);
		}
		private RegistryKey GetHive(string hiveName)
        {
            switch (hiveName)
            {
                case "HKEY_CLASSES_ROOT":
                    return Registry.ClassesRoot;
                case "HKEY_CURRENT_USER":
                    return Registry.CurrentUser;
                case "HKEY_LOCAL_MACHINE":
					return Registry.LocalMachine;
				case "HKEY_USERS":
					return Registry.Users;
				case "HKEY_CURRENT_CONFIG":
					return Registry.CurrentConfig;
			}
			return null;
		}

		private class ParseResults
		{
			private string mErrorString;
			private RegistryKey mHive;
			private string mKeyName;
			private string mValueName;

			private ParseResults() { }
			public ParseResults(string errorString)
			{
				mErrorString = errorString;
			}
			public ParseResults(RegistryKey hive, string keyName, string valueName)
			{
				mHive = hive;
				mKeyName = keyName;
				mValueName = valueName;
			}

			public bool Success
			{
				get
				{
					return mErrorString == null;
				}
			}

			public string ErrorString
			{
				get
				{
					return mErrorString;
				}
			}

			public RegistryKey Hive
			{
				get
				{
					return mHive;
				}
			}
			public string KeyName
			{
				get
				{
					return mKeyName;
				}
			}
			public string ValueName
			{
				get
				{
					return mValueName;
				}
			}
		}
	}
}
